% % % % 
% % % % 
% % % % Graphical user interface that illustrates the search process for
% % % % PSMS (Particle Swarm Model Selection)
% % % % 
% % % % Authors:    Hugo Jair Escalante,  Isabelle Guyon and Amir Saffari
% % % % Contact:    hugojair@ccc.inaoep.mx
% % % % Date:   August 23, 2008.
% % % % 
% % % %  Note: the GUI works for the FMS=1 option of PSMS
% % % % 

clear all; close all; clc;
global ud;
% % % %  Go to the directory where CLOP is located
ud.path2clop=which('use_spider_clop'); 
ud.path2clop=ud.path2clop(1:strfind(ud.path2clop, 'use_')-1)


cmd=['cd(''' ud.path2clop ''')'];eval(cmd) ;clear cmd;
% % % %  Add CLOP to the path
use_spider_clop;
% % % %  Default data set name 
ud.thisdataset='thyroid';
% % % % Directory to where the data is located
ud.datadir=[ud.path2clop '\data_mlss08\'];
ud.results_dir=[ud.path2clop '\Results\mlss08\'];

% % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % Set the default PSMS parameters % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % %
ud.itera=10;
ud.ttsize=5;
ud.redfacts=1;
ud.fmss=1;
ud.folds=2;
ud.c1=2;
ud.c2=2;
ud.model_name={'neural'};
ud.wi=1.2;
ud.we=0.4;
ud.wvf=0.5;
ud.visualize=0;
ud.viz_mods=1;

% % % % Remove all R files
% rlinkdire='C:\CLOP\challenge_objects\packages\Rlink';
% cmd=['del ' rlinkdire '\*.Rdata'];
% system(cmd);

warning off all;

% % % % Name of the data 
ud.data_name=ud.thisdataset;
% % % % Load the dataset and create the train and test sets    
ud.tdataname=[ud.datadir ud.thisdataset '\'];
da=importdata([ud.tdataname '\' ud.data_name '_train.DATA']);
db=importdata([ud.tdataname '\' ud.data_name '_train.LABELS']);
ud.D.train=data(da,db);
clear da db;
da=importdata([ud.tdataname '\' ud.data_name '_test.DATA']);
db=importdata([ud.tdataname '\' ud.data_name '_test.LABELS']);
ud.D.test=data(da,db);    
clear da db;


ud.ask = figure('Name','PSMS - CLOP v1.5','Tag','disptfig','Position',[100,100,600,500]);
set(ud.ask ,'MenuBar','none','Units','Normalized','Backingstore','off','InvertHardcopy','on',...
'PaperPositionMode','auto','resize','on','Color','white');
figcolor = get(ud.ask,'Color');

f = uimenu('Label','Data');
    uimenu(f,'Label','Load a data set',...
        'Callback',['ud.data_dir=uigetdir(ud.datadir,''Select a dataset folder'');',...
        'slashes= findstr(ud.data_dir,''\'');',...
        'lastslash=length(slashes);',...
        'ud.data_name=ud.data_dir(slashes(lastslash)+1:end);',...
        'ud.tdataname=[ud.datadir ud.data_name ''\''];',...
        'da=importdata([ud.tdataname ''\'' ud.data_name ''_train.DATA'']);',...
        'db=importdata([ud.tdataname ''\'' ud.data_name ''_train.LABELS'']);',...
        'ud.D.train=data(da,db);',...
        'clear da db;',...
        'da=importdata([ud.tdataname ''\'' ud.data_name ''_test.DATA'']);',...
        'db=importdata([ud.tdataname ''\'' ud.data_name ''_test.LABELS'']);',...
        'ud.D.test=data(da,db);',...    
        'clear da db;',...
        'ud.lb_data =uicontrol(''Style'',''text'',''Units'',''normalized'',''Position'',[.2 0.89 0.50 0.1],''ForegroundColor'',''k'',''BackgroundColor'',figcolor,''FontSize'',14,''String'',[''Data set loaded: '' ud.data_name]);',...
        ]);
    uimenu(f,'Label','Save','Callback','save');
    uimenu(f,'Label','Quit','Callback','exit',...
           'Separator','on','Accelerator','Q');

ud.lb_data =uicontrol('Style','text','Units','normalized','Position',[.2 0.89 0.50 0.1],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',14,'String',['Data set loaded: ' ud.data_name]); 
       
       
% % % % Text-box parameters first row
ud.lb_iters =uicontrol('Style','text','Units','normalized','Position',[.015 0.835 0.10 0.05],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',11,'String','Iterations '); 
ud.iters_tb =uicontrol('Style','edit','Units','normalized','Position',[.12 0.849 0.05 0.03],...
'String',num2str(ud.itera),'BackgroundColor','white',...
'CallBack',['ud.itera=str2num(get(ud.iters_tb,''String''));']);

ud.lb_ss =uicontrol('Style','text','Units','normalized','Position',[.22 0.835 0.17 0.05],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',11,'String','Swarm size '); 
ud.ss_tb =uicontrol('Style','edit','Units','normalized','Position',[.38 0.849 0.05 0.03],...
'String',num2str(ud.ttsize),'BackgroundColor','white',...
'CallBack',['ud.ttsize=str2num(get(ud.ss_tb,''String''));']);

ud.lb_fn =uicontrol('Style','text','Units','normalized','Position',[.45 0.835 0.17 0.05],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',11,'String','No. folds '); 
ud.fn_tb =uicontrol('Style','edit','Units','normalized','Position',[.6 0.849 0.05 0.03],...
'String',num2str(ud.folds),'BackgroundColor','white',...
'CallBack',['ud.folds=str2num(get(ud.fn_tb,''String''));']);

ud.sf_fn =uicontrol('Style','text','Units','normalized','Position',[.67 0.835 0.2 0.05],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',11,'String','Subs. coeff. '); 
ud.sf_tb =uicontrol('Style','edit','Units','normalized','Position',[.85 0.849 0.05 0.03],...
'String',num2str(ud.redfacts),'BackgroundColor','white',...
'CallBack',['ud.redfacts=str2num(get(ud.sf_tb,''String''));']);

% % % % Text-box parameters second row
ud.lb_c1 =uicontrol('Style','text','Units','normalized','Position',[.15 0.735 0.20 0.05],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',11,'String','c1 (local weight): '); 
ud.c1_tb =uicontrol('Style','edit','Units','normalized','Position',[.38 0.749 0.05 0.03],...
'String',num2str(ud.c1),'BackgroundColor','white',...
'CallBack',['ud.c1=str2num(get(ud.c1_tb,''String''));']);

ud.lb_c2 =uicontrol('Style','text','Units','normalized','Position',[.55 0.735 0.25 0.05],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',11,'String','c_2 (global weight) '); 
ud.c2_tb =uicontrol('Style','edit','Units','normalized','Position',[.85 0.749 0.05 0.03],...
'String',num2str(ud.c2),'BackgroundColor','white',...
'CallBack',['ud.c2=str2num(get(ud.c2_tb,''String''));']);

% % % % Text-box parameters thirf row
ud.lb_wi =uicontrol('Style','text','Units','normalized','Position',[.1 0.635 0.10 0.05],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',11,'String','W - start'); 
ud.wi_tb =uicontrol('Style','edit','Units','normalized','Position',[.22 0.649 0.05 0.03],...
'String',num2str(ud.wi),'BackgroundColor','white',...
'CallBack',['ud.wi=str2num(get(ud.wi_tb,''String''));']);

ud.lb_wvf =uicontrol('Style','text','Units','normalized','Position',[.37 0.635 0.17 0.05],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',11,'String','W - vary-for'); 
ud.wvf_tb =uicontrol('Style','edit','Units','normalized','Position',[.54 0.649 0.05 0.03],...
'String',num2str(ud.wvf),'BackgroundColor','white',...
'CallBack',['ud.wvf=str2num(get(ud.wvf_tb,''String''));']);

    ud.lb_we =uicontrol('Style','text','Units','normalized','Position',[.68 0.635 0.17 0.05],...
'ForegroundColor','k','BackgroundColor',figcolor,'FontSize',11,'String','W - end'); 
ud.we_tb =uicontrol('Style','edit','Units','normalized','Position',[.85 0.649 0.05 0.03],...
'String',num2str(ud.we),'BackgroundColor','white',...
'CallBack',['ud.we=str2num(get(ud.we_tb,''String''));']);

% % % % Run Button 
ud.trainit = uicontrol('Style','PushButton','String','Run PSMS',...
    'Units','normalized','Position',[.4,.5,.20,.1],...
'CallBack', ['algox.alg=ud.model_name{:};',...
'algox.tdat=ud.D.test;',...
'ud.dist_fig = figure(''Name'',''PSMS - CLOP v1.5'',''Tag'',''disptfig'',''Position'',[10,50,1200,780]);',...
'set(ud.dist_fig,''MenuBar'',''none'',''Units'',''Normalized'',''Backingstore'',''off'',''InvertHardcopy'',''on'',''PaperPositionMode'',''auto'',''resize'',''off'',''Color'',''white'');',...
'figcolor = get(ud.dist_fig,''Color'');',...
'g = uimenu(''Label'',''PSMS-GUI'');',...
'uimenu(g,''Label'',''Load a data set and run PSMS'',''Callback'',[''ud=load_a_dataset(ud);'']);',...
'uimenu(g,''Label'',''Set parameters for PSMS'',''Callback'',''helpdlg(''''To modify PSMS parameters: close this window and get back to the main GUI. Optionally, close all windows and run again psms_gui_dev.m'''',''''Modifying PSMS parameters'''');'');',...
'uimenu(g,''Label'',''Save figure as .fig'',''Callback'',''saveas(gcf,''''clop_psms_gui_output.fig'''');'');',...
'uimenu(g,''Label'',''Save figure as .jpeg'',''Callback'',''saveas(gcf,''''clop_psms_gui_output.jpg'''');'');',...
'uimenu(g,''Label'',''Close'',''Callback'',''close(gcf);'');',...
'h = uimenu(''Label'',''Help'');',...
'uimenu(h,''Label'',''First row plots'',''Callback'',''helpdlg(''''The left plot shows the particles in the models space. Which is a simple projection of the full-models into a 3D space; the global best model is shown with a cirlce. In the middle, it is plot the train, CV and test error of each particle as a function of number of iterations, the global best solution is enclosed within a circle. At the right, it is plot the CV error as a function of the test error; each particle is shown in a different color; the global best solution is shown within a circle.'''',''''About: First row matrices'''');'');',...
'uimenu(h,''Label'',''Second row plots'',''Callback'',''helpdlg(''''These plots show the CV-error (left) and Test-error (right) of the particles as a function of number of iterations; each particle is shown in a different color; the global best solution is enclosed within a circle. At the center are shown the CLOP-models for each particle; each model is shown in the color that corresponds to the respective particle.'''',''''About: second row matrices'''');'');',...
'uimenu(h,''Label'',''Third row plots'',''Callback'',''helpdlg(''''These matrices illustrate the selected classifier (left), feature selection method (middle) and combination of preprocessing methods (right); each color represents the methods that uses each particle.'''',''''About: third row matrices'''');'');',...
'uimenu(h,''Label'',''Modifying PSMS parameters'',''Callback'',''helpdlg(''''To modify PSMS parameters: close this window and get back to the main GUI. Optionally, close all windows and run again psms_gui_dev.m'''',''''Modifying PSMS parameters'''');'');',...
'uimenu(h,''Label'',''Preprocessing codification'',''Callback'',''helpdlg(''''000 - no preprocessing; 001 - shift-scale; 010 - standardize; 011 - standardize+shift-scale; 100 - normalize; 110 - normalize+standardize; 111 - normalize+standardize+shift-scale'''',''''Combination of preprocessing methods'''');'');',...
'algox.ud=ud;',...
'ud.X=psmsx_gui(algox,{[''viz_mods='' num2str(ud.viz_mods)],[''visualize='' num2str(ud.visualize)],[''maxiter='' num2str(ud.itera)],[''swarmsize='' num2str(ud.ttsize)],[''foldnum='' num2str(ud.folds)],[''redfactor='' num2str(ud.redfacts)],[''c1='' num2str(ud.c1)],[''c2='' num2str(ud.c2)],[''w_start='' num2str(ud.wi)],[''w_varyfor='' num2str(ud.wvf)],[''w_end='' num2str(ud.we)],[''fms='' num2str(ud.fmss)]});',...
'warning off all;',...
'ud.itime=cputime;',...   
'[ud.aa,ud.my_model]=train(ud.X,ud.D.train);',...            
'ud.timexc=cputime-ud.itime;',...
'[ud.aa,ud.bb]=train(ud.my_model.Best_model,ud.D.train);',...
'[ud.train_result]=test(ud.bb,ud.D.train);',...
'[ud.test_result]=test(ud.bb,ud.D.test);',...
'ud.ber_cv = ud.my_model.fBest;',...
'ud.ber_train=balanced_errate(ud.train_result.X, ud.train_result.Y);',...
'ud.ber_test=balanced_errate(ud.test_result.X, ud.test_result.Y);',...
'valid_tracks=find(ud.my_model.track~=100);',...
'ud.mean_cv= mean(ud.my_model.track(valid_tracks));',...
'ud.mean_train= mean(ud.my_model.tracktrain(valid_tracks));',...
'ud.mean_test= mean(ud.my_model.tracktest(valid_tracks));',...
'ud.lb_rm =uicontrol(ud.ask,''Style'',''text'',''Units'',''normalized'',''Position'',[.37 0.395 0.3 0.075],''ForegroundColor'',''k'',''BackgroundColor'',figcolor,''FontSize'',14,''String'',''Selected model'');',...
'ud.lb_rm2 =uicontrol(ud.ask,''Style'',''text'',''Units'',''normalized'',''Position'',[.1 0.32 0.8 0.1],''ForegroundColor'',''k'',''BackgroundColor'',figcolor,''FontSize'',10,''String'',ud.my_model.modelst);',...
'ud.lb_rm3 =uicontrol(ud.ask,''Style'',''text'',''Units'',''normalized'',''Position'',[.22 0.240 0.45 0.05],''ForegroundColor'',''k'',''BackgroundColor'',figcolor,''FontSize'',14,''String'',[''Training error:  '' num2str(ud.ber_train)]);',... 
'ud.lb_rm5 =uicontrol(ud.ask,''Style'',''text'',''Units'',''normalized'',''Position'',[.22 0.170 0.45 0.05],''ForegroundColor'',''k'',''BackgroundColor'',figcolor,''FontSize'',14,''String'',[''CV  error:       '' num2str(ud.ber_cv)]);',... 
'ud.lb_rm7 =uicontrol(ud.ask,''Style'',''text'',''Units'',''normalized'',''Position'',[.22 0.100 0.45 0.05],''ForegroundColor'',''k'',''BackgroundColor'',figcolor,''FontSize'',14,''String'',[''Test-error:      '' num2str(ud.ber_test)]);',... 
'warning on all;',...
]);
% ''helpdlg(''''To modify PSMS parameters: close this window and get back to the main GUI. Optionally, close all windows and run again psms_gui_dev.m'''',''''Modifying PSMS parameters'''');'',
% 'uimenu(g,''Label'',''Save figure as jpg'',''Callback'',''saveas(gcf,''output'',''fig'');'');',...
% 
% 
% 'uimenu(f,''Label'',''Save figure as jpg'',''Callback'',''saveas(gcf,''clop_psms_gui.fig'');'');',...
% 'f = uimenu(''Label'',''Help'');',...
% 'uimenu(f,''Label'',''Preprocessing codification'',''Callback'',''save;'');',...
